function [wp,wm,H1,H2,HH1,HH2,G,Gh]=dualtreeM2D_freq(x,M,rm)

%function [wp,wm,H1,H2,HH1,HH2,G,Gh]=dualtreeM2D_freq(x,M,rm)
%
%Complete function for the 2D dual-tree M-band transform
%INPUT:
%   x : image to decompose. x=x(1:M1^rm*floor(size(x,1)/M1^rm),1:M2^rm*floor(size(x,2)/M2^rm));
%       the image is troncated: M1 being the number of bands of H1 and M2
%       being the number of bands of H2
%   M : filters to use. Ex: M=2 for Meyer 2 bands, M='article' for Alkin
%   and Caglar ...
%   rm : decomposition level
%OUTPUT:
%   wp: coefficients issued from the primal tree (including processing)
%   wm: coefficients issued from the dual tree (including processing)
%   H1: column filters
%   H2: row filters
%   HH1: column dual filters
%   HH2: row dual filters
%   G: primal pre-filter (2D)
%   Gh: dual prefilter (2D)

%*****************************************************
%* author: Caroline Chaux                            *
%* institution: Institut Gaspard Monge - CNRS	     *
%*	       Universit de Marne la Valle             *
%* date: Thursday, October, 6th                      *
%*****************************************************


if M==2
    %Meyer 2 bands
    M1=2;M2=2;
    ep=1/5;
    x=x(1:M1^rm*floor(size(x,1)/M1^rm),1:M2^rm*floor(size(x,2)/M2^rm));
    [LX1,LX2] = size(x);
    om1 = (0:LX1-1)*2*pi/LX1;
    om2 = (0:LX2-1)*2*pi/LX2;
    H1 = fr2bandmeyer(om1,ep);
    H2 = fr2bandmeyer(om2,ep);
    m='M2';
elseif M == 3
    %Meyer 3 bands
    M1=3;M2=3;
    ep = 1/7;
    x=x(1:M1^rm*floor(size(x,1)/M1^rm),1:M2^rm*floor(size(x,2)/M2^rm));
    [LX1,LX2] = size(x);
    om1 = (0:LX1-1)*2*pi/LX1;
    om2 = (0:LX2-1)*2*pi/LX2;
    H1 = fr3bandmeyer(om1,ep);
    H2 = fr3bandmeyer(om2,ep);
    m='M3';
elseif M == 4
    %Meyer 4 bands
    M1=4;M2=4;
    ep = 1/9;
    x=x(1:M1^rm*floor(size(x,1)/M1^rm),1:M2^rm*floor(size(x,2)/M2^rm));
    [LX1,LX2] = size(x);
    om1 = (0:LX1-1)*2*pi/LX1;
    om2 = (0:LX2-1)*2*pi/LX2;
    H1 = fr4bandmeyer(om1,ep);
    H2 = fr4bandmeyer(om2,ep);
    m='M4';
elseif strcmp(M,'article')
    %Alkin and Caglar 4 bands
    M1=4;M2=4;
    [af,sf]=filters('article');
    x=x(1:M1^rm*floor(size(x,1)/M1^rm),1:M2^rm*floor(size(x,2)/M2^rm));
    [LX1,LX2] = size(x);
    af1=[af(size(af,1)/2+1:end,:);zeros(LX1-size(af,1),4);af(1:size(af,1)/2,:)];
    af2=[af(size(af,1)/2+1:end,:);zeros(LX2-size(af,1),4);af(1:size(af,1)/2,:)];
    H1 = (fft(af1)).';
    H2 = (fft(af2)).';
    m='article';
elseif strcmp(M,'genlot16_02')
    M1=16;M2=16;
    [af,sf]=filters('genlot16_02');
    x=x(1:M1^rm*floor(size(x,1)/M1^rm),1:M2^rm*floor(size(x,2)/M2^rm));
    [LX1,LX2] = size(x);
    af1=[af(size(af,1)/2+1:end,:);zeros(LX1-size(af,1),16);af(1:size(af,1)/2,:)];
    af2=[af(size(af,1)/2+1:end,:);zeros(LX2-size(af,1),16);af(1:size(af,1)/2,:)];
    H1 = (fft(af1)).';
    H2 = (fft(af2)).';
    m='genlot16_02';
elseif strcmp(M,'genlot08_02')
    M1=8;M2=8;
    [af,sf]=filters('genlot08_02');
    x=x(1:M1^rm*floor(size(x,1)/M1^rm),1:M2^rm*floor(size(x,2)/M2^rm));
    [LX1,LX2] = size(x);
    af1=[af(size(af,1)/2+1:end,:);zeros(LX1-size(af,1),8);af(1:size(af,1)/2,:)];
    af2=[af(size(af,1)/2+1:end,:);zeros(LX2-size(af,1),8);af(1:size(af,1)/2,:)];
    H1 = (fft(af1)).';
    H2 = (fft(af2)).';
    m='genlot08_02';
elseif strcmp(M,'genlot08_04')
    M1=8;M2=8;
    [af,sf]=filters('genlot08_04');
    x=x(1:M1^rm*floor(size(x,1)/M1^rm),1:M2^rm*floor(size(x,2)/M2^rm));
    [LX1,LX2] = size(x);
    af1=[af(size(af,1)/2+1:end,:);zeros(LX1-size(af,1),8);af(1:size(af,1)/2,:)];
    af2=[af(size(af,1)/2+1:end,:);zeros(LX2-size(af,1),8);af(1:size(af,1)/2,:)];
    H1 = (fft(af1)).';
    H2 = (fft(af2)).';
    m='genlot08_04';
elseif strcmp(M,'genlot08_06')
    M1=8;M2=8;
    [af,sf]=filters('genlot08_06');
    x=x(1:M1^rm*floor(size(x,1)/M1^rm),1:M2^rm*floor(size(x,2)/M2^rm));
    [LX1,LX2] = size(x);
    af1=[af(size(af,1)/2+1:end,:);zeros(LX1-size(af,1),8);af(1:size(af,1)/2,:)];
    af2=[af(size(af,1)/2+1:end,:);zeros(LX2-size(af,1),8);af(1:size(af,1)/2,:)];
    H1 = (fft(af1)).';
    H2 = (fft(af2)).';
    m='genlot08_06';
elseif strcmp(M,'MLT16')
    %Lapped transform 16 bands
    M1=16;M2=16;
    [af,sf]=filters('MLT16');
    x=x(1:M1^rm*floor(size(x,1)/M1^rm),1:M2^rm*floor(size(x,2)/M2^rm));
    [LX1,LX2] = size(x);
    af1=[af(size(af,1)/2+1:end,:);zeros(LX1-size(af,1),16);af(1:size(af,1)/2,:)];
    af2=[af(size(af,1)/2+1:end,:);zeros(LX2-size(af,1),16);af(1:size(af,1)/2,:)];
    H1 = (fft(af1)).';
    H2 = (fft(af2)).';
    m='MLT16';
elseif strcmp(M,'MLT8')
    %Lapped transform 8 bands
    M1=8;M2=8;
    [af,sf]=filters('MLT8');
    x=x(1:M1^rm*floor(size(x,1)/M1^rm),1:M2^rm*floor(size(x,2)/M2^rm));
    [LX1,LX2] = size(x);
    af1=[af(size(af,1)/2+1:end,:);zeros(LX1-size(af,1),8);af(1:size(af,1)/2,:)];
    af2=[af(size(af,1)/2+1:end,:);zeros(LX2-size(af,1),8);af(1:size(af,1)/2,:)];
    H1 = (fft(af1)).';
    H2 = (fft(af2)).';
    m='MLT8';
elseif strcmp(M,'MLT4')
    %Lapped transform 4 bands
    M1=4;M2=4;
    [af,sf]=filters('MLT4');
    x=x(1:M1^rm*floor(size(x,1)/M1^rm),1:M2^rm*floor(size(x,2)/M2^rm));
    [LX1,LX2] = size(x);
    af1=[af(size(af,1)/2+1:end,:);zeros(LX1-size(af,1),4);af(1:size(af,1)/2,:)];
    af2=[af(size(af,1)/2+1:end,:);zeros(LX2-size(af,1),4);af(1:size(af,1)/2,:)];
    H1 = (fft(af1)).';
    H2 = (fft(af2)).';
    m='MLT4';
elseif strcmp(M,'MLT3')
    %Lapped transform 3 bands
    M1=3;M2=3;
    [af,sf]=filters('MLT3');
    x=x(1:M1^rm*floor(size(x,1)/M1^rm),1:M2^rm*floor(size(x,2)/M2^rm));
    [LX1,LX2] = size(x);
    af1=[af(size(af,1)/2+1:end,:);zeros(LX1-size(af,1),3);af(1:size(af,1)/2,:)];
    af2=[af(size(af,1)/2+1:end,:);zeros(LX2-size(af,1),3);af(1:size(af,1)/2,:)];
    H1 = (fft(af1)).';
    H2 = (fft(af2)).';
    m='MLT3';
elseif strcmp(M,'symlets')
    %Symlets 2 bands (length 8)
    M1=2;M2=2;
    [af,sf]=filters('symlets');
    x=x(1:M1^rm*floor(size(x,1)/M1^rm),1:M2^rm*floor(size(x,2)/M2^rm));
    [LX1,LX2] = size(x);
    af1=[af(size(af,1)/2+1:end,:);zeros(LX1-size(af,1),2);af(1:size(af,1)/2,:)];
    af2=[af(size(af,1)/2+1:end,:);zeros(LX2-size(af,1),2);af(1:size(af,1)/2,:)];
    H1 = (fft(af1)).';
    H2 = (fft(af2)).';
    m='symlets';
elseif strcmp(M,'symlets8')
    %Symlets 2 bands (length 16)
    M1=2;M2=2;
    [af,sf]=filters('symlets8');
    x=x(1:M1^rm*floor(size(x,1)/M1^rm),1:M2^rm*floor(size(x,2)/M2^rm));
    [LX1,LX2] = size(x);
    af1=[af(size(af,1)/2+1:end,:);zeros(LX1-size(af,1),2);af(1:size(af,1)/2,:)];
    af2=[af(size(af,1)/2+1:end,:);zeros(LX2-size(af,1),2);af(1:size(af,1)/2,:)];
    H1 = (fft(af1)).';
    H2 = (fft(af2)).';
    m='symlets8';
elseif strcmp(M,'haar')
    M1=2;M2=2;
    [af,sf]=filters('haar');
    x=x(1:M1^rm*floor(size(x,1)/M1^rm),1:M2^rm*floor(size(x,2)/M2^rm));
    [LX1,LX2] = size(x);
    af1=[sf;zeros(LX1-size(af,1),2)];
    af2=[sf;zeros(LX2-size(af,1),2)];
    H1 = (fft(af1)).';
    H2 = (fft(af2)).';
    m='haar';
elseif strcmp(M,'db4')
    %Daubechies 2 bands (length 8)
    M1=2;M2=2;
    [af,sf]=filters('db4');
    x=x(1:M1^rm*floor(size(x,1)/M1^rm),1:M2^rm*floor(size(x,2)/M2^rm));
    [LX1,LX2] = size(x);
    af1=[af(size(af,1)/2+1:end,:);zeros(LX1-size(af,1),2);af(1:size(af,1)/2,:)];
    af2=[af(size(af,1)/2+1:end,:);zeros(LX2-size(af,1),2);af(1:size(af,1)/2,:)];
    H1 = (fft(af1)).';
    H2 = (fft(af2)).';
    m='db4';
end



%normalisation
H1 = H1/sqrt(M1);
H2 = H2/sqrt(M2);

%Definition of Hilbert pairs
[HH1,HH2]=hilbert_filters_freq(H1,H2);

if rem(LX1,M1^rm) ~= 0 | rem(LX2,M2^rm) ~= 0
    error('Dimensions should be multiple of M2 or M1 ^rm');
end


%Prefiltering
[Gl,Gc,Ghl,Ghc]=genere_gamma2_freq(M1,M2,m,LX1,LX2);


G=Gl.'*Gc;
Gh=Ghl.'*Ghc;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%normalisation
x = fft2(x)/sqrt(LX1*LX2);


%prefiltering
N=(Gl.'*Gc).*x;
Nh=(Ghl.'*Ghc).*x;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%first tree "normal"
LX1r = LX1;
LX2r = LX2;

for r = 1:rm
    %for m1 = 1:M
    %    ytemp{r}{m1} = zeros(LX1r,LX2r/M)
    %end
    ytemp = zeros(M2,LX1r,LX2r/M2);

    ind = 1:M2^(r-1):LX2;
    HH = conj(H2(:,ind));
    for n = 1:LX1r
        Ytemp = analMDFT(N(n,:),HH);
        for m2 = 1:M2
            ytemp(m2,n,:) = Ytemp(m2,:);
        end
    end

    ind = 1:M1^(r-1):LX1;
    HH = conj(H1(:,ind));

    for m1 = 1:M1
        for m2 = 1:M2
            y{r}{m1}{m2} = zeros(LX1r/M1,LX2r/M2);
        end
    end

    LX2r = LX2r/M2;
    for m2 = 1:M2
        for n = 1:LX2r
            Ytemp = analMDFT(ytemp(m2,:,n),HH);
            for m1 = 1:M1
                y{r}{m1}{m2}(:,n) = Ytemp(m1,:).';
            end
        end
    end

    LX1r = LX1r/M1;
    for m1 = 1:M1
        for m2 = 1:M2
            if m1 == 1 & m2 == 1
                N = y{r}{1}{1};
                y{r}{1}{1} = [];
            else
                y{r}{m1}{m2} = real(ifft2(y{r}{m1}{m2})*sqrt(LX1r*LX2r));
            end
        end
    end

end
y{rm}{1}{1} = real(ifft2(N)*sqrt(LX1r*LX2r));


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%second tree "dual"
LX1r = LX1;
LX2r = LX2;

for r = 1:rm
    %for m1 = 1:M
    %    ytemp{r}{m1} = zeros(LX1r,LX2r/M)
    %end
    ytemph = zeros(M2,LX1r,LX2r/M2);

    ind = 1:M2^(r-1):LX2;
    HHH = conj(HH2(:,ind));
    for n = 1:LX1r
        Ytemph = analMDFT(Nh(n,:),HHH);
        for m2 = 1:M2
            ytemph(m2,n,:) = Ytemph(m2,:);
        end
    end

    ind = 1:M1^(r-1):LX1;
    HHH = conj(HH1(:,ind));

    for m1 = 1:M1
        for m2 = 1:M2
            yh{r}{m1}{m2} = zeros(LX1r/M1,LX2r/M2);
        end
    end

    LX2r = LX2r/M2;
    for m2 = 1:M2
        for n = 1:LX2r
            Ytemph = analMDFT(ytemph(m2,:,n),HHH);
            for m1 = 1:M1
                yh{r}{m1}{m2}(:,n) = Ytemph(m1,:).';
            end
        end
    end

    LX1r = LX1r/M1;
    for m1 = 1:M1
        for m2 = 1:M2
            if m1 == 1 & m2 == 1
                Nh = yh{r}{1}{1};
                yh{r}{1}{1} = [];
            else
                yh{r}{m1}{m2} = real(ifft2(yh{r}{m1}{m2})*sqrt(LX1r*LX2r));
            end
        end
    end

end
yh{rm}{1}{1} = real(ifft2(Nh)*sqrt(LX1r*LX2r));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Last treatment : the rotation
[wp,wm]=dualtree_Mband2Dcompleted(y,yh,rm);


%"unnormalization" if the filters are used
H1 = H1*sqrt(M1);
H2 = H2*sqrt(M2);
HH1 = HH1*sqrt(M1);
HH2 = HH2*sqrt(M2);

