function [Phi1,Phi2]=genere_phi_freq(Ml,Mc,m,Nl,Nc)

%[Phi1,Phi2]=genere_phi_freq(Ml,Mc,m,Nl,Nc)
%Genere the scaling function Phi in frequential (column(1) and row(2))
%INPUT:
%   rm : used number of decomposition levels to compute filters
%   Ml,Mc : bands numbers  column and row
%   m: filters used (see dualtreeM2D_freq.m)
%   Nl,Nc : image's number points,  column and row
%OUTPUT :
%   Phi1 : column scaling function
%   Phi2 : row scaling function

%*****************************************************
%* author: Caroline Chaux                            *
%* institution: Institut Gaspard Monge - CNRS	     *
%*	       Universit de Marne la Valle	         *
%* date: Thursday, October, 6th                      *
%*****************************************************

jmax=3;

Tl=Nl*Ml^(jmax-1);
Tc=Nc*Mc^(jmax-1);

oml = (0:Tl-1)*2*pi/Tl;
omc = (0:Tc-1)*2*pi/Tc;

switch m
    case 'M2'
        ep=1/5;
        H1 = fr2bandmeyer(oml,ep);
        H2 = fr2bandmeyer(omc,ep);
    case 'M3'
        ep = 1/7;
        H1 = fr3bandmeyer(oml,ep);
        H2 = fr3bandmeyer(omc,ep);
    case 'M4'
        ep = 1/9;
        H1 = fr4bandmeyer(oml,ep);
        H2 = fr4bandmeyer(omc,ep);
    case 'symlets'
        [af,sf]=filters('symlets');
        af1=[af(size(af,1)/2+1:end,:);zeros(Tl-size(af,1),2);af(1:size(af,1)/2,:)];
        af2=[af(size(af,1)/2+1:end,:);zeros(Tc-size(af,1),2);af(1:size(af,1)/2,:)];
        H1 = (fft(af1)).';
        H2 = (fft(af2)).';
    case 'symlets8'
        [af,sf]=filters('symlets8');
        af1=[af(size(af,1)/2+1:end,:);zeros(Tl-size(af,1),2);af(1:size(af,1)/2,:)];
        af2=[af(size(af,1)/2+1:end,:);zeros(Tc-size(af,1),2);af(1:size(af,1)/2,:)];
        H1 = (fft(af1)).';
        H2 = (fft(af2)).';
    case 'haar'
        [af,sf]=filters('haar');
        af1=[af(size(af,1)/2+1:end,:);zeros(Tl-size(af,1),2);af(1:size(af,1)/2,:)];
        af2=[af(size(af,1)/2+1:end,:);zeros(Tc-size(af,1),2);af(1:size(af,1)/2,:)];
        H1 = (fft(af1)).';
        H2 = (fft(af2)).';
    case 'db4'
        [af,sf]=filters('db4');
        af1=[af(size(af,1)/2+1:end,:);zeros(Tl-size(af,1),2);af(1:size(af,1)/2,:)];
        af2=[af(size(af,1)/2+1:end,:);zeros(Tc-size(af,1),2);af(1:size(af,1)/2,:)];
        H1 = (fft(af1)).';
        H2 = (fft(af2)).';
    case 'article'
        [af,sf]=filters('article');
        af1=[af(size(af,1)/2+1:end,:);zeros(Tl-size(af,1),4);af(1:size(af,1)/2,:)];
        af2=[af(size(af,1)/2+1:end,:);zeros(Tc-size(af,1),4);af(1:size(af,1)/2,:)];
        H1 = (fft(af1)).';
        H2 = (fft(af2)).';
    case 'genlot16_02'
        [af,sf]=filters('genlot16_02');
        af1=[af(size(af,1)/2+1:end,:);zeros(Tl-size(af,1),16);af(1:size(af,1)/2,:)];
        af2=[af(size(af,1)/2+1:end,:);zeros(Tc-size(af,1),16);af(1:size(af,1)/2,:)];
        H1 = (fft(af1)).';
        H2 = (fft(af2)).';
    case 'genlot08_02'
        [af,sf]=filters('genlot08_02');
        af1=[af(size(af,1)/2+1:end,:);zeros(Tl-size(af,1),8);af(1:size(af,1)/2,:)];
        af2=[af(size(af,1)/2+1:end,:);zeros(Tc-size(af,1),8);af(1:size(af,1)/2,:)];
        H1 = (fft(af1)).';
        H2 = (fft(af2)).';
    case 'genlot08_04'
        [af,sf]=filters('genlot08_04');
        af1=[af(size(af,1)/2+1:end,:);zeros(Tl-size(af,1),8);af(1:size(af,1)/2,:)];
        af2=[af(size(af,1)/2+1:end,:);zeros(Tc-size(af,1),8);af(1:size(af,1)/2,:)];
        H1 = (fft(af1)).';
        H2 = (fft(af2)).';
    case 'genlot08_06'
        [af,sf]=filters('genlot08_06');
        af1=[af(size(af,1)/2+1:end,:);zeros(Tl-size(af,1),8);af(1:size(af,1)/2,:)];
        af2=[af(size(af,1)/2+1:end,:);zeros(Tc-size(af,1),8);af(1:size(af,1)/2,:)];
        H1 = (fft(af1)).';
        H2 = (fft(af2)).';
    case 'MLT16'
        [af,sf]=filters('MLT16');
        af1=[af(size(af,1)/2+1:end,:);zeros(Tl-size(af,1),16);af(1:size(af,1)/2,:)];
        af2=[af(size(af,1)/2+1:end,:);zeros(Tc-size(af,1),16);af(1:size(af,1)/2,:)];
        H1 = (fft(af1)).';
        H2 = (fft(af2)).';
    case 'MLT8'
        [af,sf]=filters('MLT8');
        af1=[af(size(af,1)/2+1:end,:);zeros(Tl-size(af,1),8);af(1:size(af,1)/2,:)];
        af2=[af(size(af,1)/2+1:end,:);zeros(Tc-size(af,1),8);af(1:size(af,1)/2,:)];
        H1 = (fft(af1)).';
        H2 = (fft(af2)).';
    case 'MLT4'
        [af,sf]=filters('MLT4');
        af1=[af(size(af,1)/2+1:end,:);zeros(Tl-size(af,1),4);af(1:size(af,1)/2,:)];
        af2=[af(size(af,1)/2+1:end,:);zeros(Tc-size(af,1),4);af(1:size(af,1)/2,:)];
        H1 = (fft(af1)).';
        H2 = (fft(af2)).';
    case 'MLT3'
        [af,sf]=filters('MLT3');
        af1=[af(size(af,1)/2+1:end,:);zeros(Tl-size(af,1),3);af(1:size(af,1)/2,:)];
        af2=[af(size(af,1)/2+1:end,:);zeros(Tc-size(af,1),3);af(1:size(af,1)/2,:)];
        H1 = (fft(af1)).';
        H2 = (fft(af2)).';
end

H01=H1(1,:).*(1/sqrt(Ml));
Phi1=ones(1,Nl);

for j=1:jmax,
    Phi1=Phi1.*H01(1:Ml^(jmax-j):Nl*Ml^(jmax-j));
end

H02=H2(1,:).*(1/sqrt(Mc));
Phi2=ones(1,Nc);

for j=1:jmax,
    Phi2=Phi2.*H02(1:Mc^(jmax-j):Nc*Mc^(jmax-j));
end

% figure
% plot((0:Nl-1)*Ml*2*pi/Nl,abs(Phi1))
%
% figure
% plot((0:Nc-1)*Mc*2*pi/Nc,abs(Phi2))

