function xrec = idualtreeM2D_freq(wp,wm,rm,H1,H2,HH1,HH2,G,Gh)

%function xrec = idualtreeM2D_freq(wp,wm,rm,H1,H2,HH1,HH2,G,Gh)
%
%Inverse dual-tree transform
%INPUT: 
%   wp: coefficients issued from the primal tree (including processing)
%   wm: coefficients issued from the dual tree (including processing)
%   rm: decomposition level
%   H1: column filters
%   H2: row filters
%   HH1: column dual filters
%   HH2: row dual filters
%   G: primal pre-filter (2D)
%   Gh: dual prefilter (2D)
%OUTPUT:
%   xrec: reconstructed image

%*****************************************************
%* author: Caroline Chaux                            *
%* institution: Institut Gaspard Monge - CNRS	     *
%*	       Universit de Marne la Valle    	     *
%* date: Thursday, October, 6th                      *
%*****************************************************


%Inverse rotation
[y,yh]=idualtree_Mband2Dcompleted(wp,wm,rm);


M1 = size(H1,1);
M2 = size(H2,1);


%normalisation
H1 = H1/sqrt(M1);
H2 = H2/sqrt(M2);
HH1 = HH1/sqrt(M1);
HH2 = HH2/sqrt(M2);



[LX1,LX2] = size(y{1}{1}{2});
LX1 = LX1*M1;
LX2 = LX2*M2;


%Reconstruction of first tree
x = y{rm}{1}{1};
[LX1r,LX2r] = size(x);
%x = y{rm}{1}{1};
%%[LX1r,LX2r] = size(y{rm}{1}{1});
x = fft2(x)/sqrt(LX1r*LX2r);
for r = rm:-1:1
    for m1 = 1:M1
        for m2 = 1:M2
            if m1 ~= 1 | m2 ~=1
                y{r}{m1}{m2} = fft2(y{r}{m1}{m2})/sqrt(LX1r*LX2r);
            end
        end
    end
    
    ind = 1:M1^(r-1):LX1;
    HH = H1(:,ind);
    
    LX1r = M1*LX1r;
    ytemp = zeros(M1,LX1r,LX2r);
    for m2 = 1:M2
        for n = 1:LX2r
            if m2 == 1
                Ytemp = x(:,n).';
            else
                Ytemp = (y{r}{1}{m2}(:,n)).';
            end
            for m1 = 2:M1
                Ytemp = [Ytemp;(y{r}{m1}{m2}(:,n)).'];
            end
            ytemp(m2,:,n) = synthMDFT(Ytemp,HH);
        end
    end
    
    
    ind = 1:M2^(r-1):LX2;
    HH = H2(:,ind);
    
    LX2r = LX2r*M2;    
    x = zeros(LX1r,LX2r);
    for n = 1:LX1r
        x(n,:) = synthMDFT(squeeze(ytemp(:,n,:)),HH);
    end
    
    %if r ~= 1
    %    y{r-1}{1}{1} = x;
    %end
    
end



%Reconstruction of second tree
xh = yh{rm}{1}{1};
[LX1r,LX2r] = size(xh);
%x = y{rm}{1}{1};
%%[LX1r,LX2r] = size(y{rm}{1}{1});
xh = fft2(xh)/sqrt(LX1r*LX2r);
for r = rm:-1:1
    for m1 = 1:M1
        for m2 = 1:M2
            if m1 ~= 1 | m2 ~=1
                yh{r}{m1}{m2} = fft2(yh{r}{m1}{m2})/sqrt(LX1r*LX2r);
            end
        end
    end
    
    ind = 1:M1^(r-1):LX1;
    HHH = HH1(:,ind);
    
    LX1r = M1*LX1r;
    ytemph = zeros(M1,LX1r,LX2r);
    for m2 = 1:M2
        for n = 1:LX2r
            if m2 == 1
                Ytemph = xh(:,n).';
            else
                Ytemph = (yh{r}{1}{m2}(:,n)).';
            end
            for m1 = 2:M1
                Ytemph = [Ytemph;(yh{r}{m1}{m2}(:,n)).'];
            end
            ytemph(m2,:,n) = synthMDFT(Ytemph,HHH);
        end
    end
    
    
    ind = 1:M2^(r-1):LX2;
    HHH = HH2(:,ind);
    
    LX2r = LX2r*M2;    
    xh = zeros(LX1r,LX2r);
    for n = 1:LX1r
        xh(n,:) = synthMDFT(squeeze(ytemph(:,n,:)),HHH);
    end
    
    %if r ~= 1
    %    y{r-1}{1}{1} = x;
    %end
    
end



%Iprefiltering
N=conj(G).*x;
Nh=conj(Gh).*xh;

%pseudo-inverse
Xrec=(N+Nh)./((abs(G)).^2+(abs(Gh)).^2);

xrec=real(ifft2(Xrec)*sqrt(LX1*LX2));

